import PyLibrary.funciones_comunes as fc
import pandas as pd
import os
from translation_config import TranslationConfig
from openpyxl import load_workbook
from openpyxl.styles import Font

# Definir el logger a nivel de módulo
logger = None


def importar_traduccion(config: TranslationConfig):
    archivo_maestro = config.get_master_path()
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    master_col = config.codigo_idioma_seleccionado
    translated_col = f"{config.codigo_idioma_seleccionado} Translated"
    affinity_col = f"{config.codigo_idioma_seleccionado} Affinity"

    df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
    df_traduccion = fc.read_dataframe_with_cleanup_retries(
        config.get_auto_translate_path()
    )

    celdas_modificadas = {}

    for index, fila in df_traduccion.iterrows():
        clave = fila[df_maestro.columns[0]]
        if clave in df_maestro[df_maestro.columns[0]].values:
            if (
                fila[affinity_col] >= config.nivel_afinidad_minimo
                and pd.notnull(fila[translated_col])
                and fila[translated_col] != ""
            ):
                valor_traducido = fila[translated_col]
                valor_original = df_maestro.loc[
                    df_maestro[df_maestro.columns[0]] == clave, master_col
                ].values[0]

                if str(valor_original) != str(valor_traducido):
                    df_maestro.loc[
                        df_maestro[df_maestro.columns[0]] == clave, master_col
                    ] = valor_traducido
                    logger.info(
                        f'Fila {index}, Columna {translated_col}: "{valor_original}" actualizado a "{valor_traducido}"'
                    )

                    fila_excel = (
                        df_maestro.index[
                            df_maestro[df_maestro.columns[0]] == clave
                        ].tolist()[0]
                        + 2
                    )
                    columna_excel = df_maestro.columns.get_loc(master_col) + 1
                    celdas_modificadas[(fila_excel, columna_excel)] = valor_traducido

    fc.save_dataframe_with_retries(df_maestro, output_path=archivo_maestro)

    aplicar_negrita_celdas_modificadas(archivo_maestro, celdas_modificadas)

    print(f"Traducciones importadas y archivo maestro actualizado: {archivo_maestro}.")


def aplicar_negrita_celdas_modificadas(archivo, celdas_modificadas):
    workbook = load_workbook(archivo)
    sheet = workbook.active

    for (fila, columna), valor in celdas_modificadas.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(bold=True)
        celda.value = valor

    workbook.save(archivo)
    print(
        f"Se han marcado en negrita {len(celdas_modificadas)} celdas modificadas en {archivo}."
    )


def run(config: TranslationConfig):
    global logger
    logger = fc.configurar_logger(config.work_dir)
    importar_traduccion(config)


if __name__ == "__main__":
    import menu_pasos_traduccion

    menu_pasos_traduccion.main()
